include(CheckCXXSourceCompiles)

set(CMAKE_REQUIRED_FLAGS "-pedantic-errors")

# fold expressions (a + ...)
check_cxx_source_compiles("
  template <class... Args>
  int f(Args... args)
  {
    return (args + ...);
  }
  int main()
  {
    f(0,1,2,3,4,5);
  }
"  AMDIS_HAS_CXX_FOLD_EXPRESSIONS
)

check_cxx_source_compiles("
  template <int I>
  auto f()
  {
    if constexpr(I == 0)
      return double{1.0};
    else
      return int{0};
  }
  int main()
  {
    return f<1>();
  }
"  AMDIS_HAS_CXX_CONSTEXPR_IF
)

check_cxx_source_compiles("
  #include <iostream>
  #include <tuple>
  int main()
  {
    auto tup = std::make_tuple(0, 'a', 3.14);
    for... (auto elem : tup)
      std::cout << elem << std::endl;
  }
"  AMDIS_HAS_EXPANSION_STATEMENTS
)

check_cxx_source_compiles("
  template <auto n>
  struct Foo {};
  int main()
  {
    Foo<0> foo_int;
    Foo<true> foo_bool;
    Foo<1u> foo_unsigned;
  }
"  AMDIS_HAS_CXX_AUTO_TEMPLATE_PARAMETER
)

unset(CMAKE_REQUIRED_FLAGS)