# FindHYPRE.cmake
#
# Finds the HYPRE library
#
# This will define the following variables
#
#    HYPRE_FOUND
#    HYPRE_INCLUDE_DIR
#    HYPRE_LIBRARIES
#
# and the following imported targets
#
#     HYPRE::HYPRE
#
# Author: Simon Praetorius <simon.praetorius@tu-dresden.de>


mark_as_advanced(HYPRE_FOUND HYPRE_INCLUDE_DIR HYPRE_LIBRARIES)

find_path(HYPRE_INCLUDE_DIR HYPRE.h
  PATHS
    /opt/software/hypre
    ${HYPRE_DIR} ${HYPRE_ROOT}
    ENV HYPRE_DIR ENV HYPRE_ROOT
    ENV EBROOTHYPRE
  PATH_SUFFIXES
    hypre include include/hypre
  NO_DEFAULT_PATH)
find_path(HYPRE_INCLUDE_DIR HYPRE.h
  PATH_SUFFIXES hypre include include/hypre)

find_library(HYPRE_LIBRARY HYPRE
  PATHS
    /opt/software/hypre
    ${HYPRE_DIR} ${HYPRE_ROOT}
    ENV HYPRE_DIR ENV HYPRE_ROOT
    ENV EBROOTHYPRE
  PATH_SUFFIXES
    lib
  NO_DEFAULT_PATH)
find_library(HYPRE_LIBRARY HYPRE
  PATH_SUFFIXES lib)

if (HYPRE_LIBRARY)
  set(HYPRE_LIBRARIES ${HYPRE_LIBRARY})
  get_filename_component(HYPRE_LIBRARY_DIR ${HYPRE_LIBRARY} DIRECTORY)

  foreach(_lib_name "HYPRE_FEI" "HYPRE_core")
    find_library(_lib ${_lib_name} HINTS ${HYPRE_LIBRARY_DIR} NO_DEFAULT_PATH)
    if (_lib)
      list(APPEND HYPRE_LIBRARIES ${_lib})
    endif (_lib)
    unset(_lib)
  endforeach()
  unset(_lib_name)
endif (HYPRE_LIBRARY)

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(HYPRE
  REQUIRED_VARS HYPRE_INCLUDE_DIR HYPRE_LIBRARY
)

# text for feature summary
set_package_properties("HYPRE" PROPERTIES
  DESCRIPTION "high performance preconditioners")


if(HYPRE_FOUND AND NOT TARGET HYPRE::HYPRE)
  add_library(HYPRE::HYPRE INTERFACE IMPORTED)

  set_target_properties(HYPRE::HYPRE PROPERTIES
    INTERFACE_INCLUDE_DIRECTORIES "${HYPRE_INCLUDE_DIR}"
    INTERFACE_LINK_LIBRARIES "${HYPRE_LIBRARIES}"
  )
endif()
