#pragma once

#include <utility>

#include <amdis/Assembler.hpp>
#include <amdis/LocalOperator.hpp>
#include <amdis/operations/Assigner.hpp>
#include <amdis/typetree/Traversal.hpp>
#include <amdis/utility/AssembleOperators.hpp>

namespace AMDiS {

template <class GB, class B>
  template <class ContextTag, class Expr, class TreePath>
void LinearForm<GB,B>::
addOperator(ContextTag contextTag, Expr const& expr, TreePath path)
{
  static_assert( Concepts::PreTreePath<TreePath>,
      "path must be a valid treepath, or an integer/index-constant");

  auto i = child(this->basis()->localView().tree(), makeTreePath(path));

  using LocalContext = typename ContextTag::type;
  using Traits = DefaultAssemblerTraits<LocalContext, ElementVector>;
  auto op = makeLocalOperator<LocalContext>(expr, this->basis()->gridView());
  auto localAssembler = makeUniquePtr(makeAssembler<Traits>(std::move(op), i));

  operators_[i].push(contextTag, std::move(localAssembler));
}


template <class GB, class B>
void LinearForm<GB,B>::
assemble(typename GB::LocalView const& localView)
{
  elementVector_.resize(localView.size());
  elementVector_ = 0;

  auto const& gv = this->basis()->gridView();
  auto const& element = localView.element();
  auto geometry = element.geometry();

  for_each_node(localView.tree(), [&](auto const& node, auto) {
    auto& rhsOp = operators_[node];
    if (rhsOp) {
      rhsOp.bind(element, geometry);
      assembleOperators(gv, element, rhsOp, makeVectorAssembler(node, elementVector_));
      rhsOp.unbind();
    }
  });

  this->scatter(localView, elementVector_, Assigner::plus_assign{});
}


template <class GB, class B>
void LinearForm<GB,B>::
assemble()
{
  auto localView = this->basis()->localView();

  this->init(true);
  for (auto const& element : elements(this->basis()->gridView(), typename Traits::PartitionSet{})) {
    localView.bind(element);
    this->assemble(localView);
    localView.unbind();
  }
  this->finish();
}

} // end namespace AMDiS
