#pragma once

#include <dune/common/version.hh>

namespace AMDiS
{
  // dune version independent extraction of node type from preBasis
  template <class PB, class TP>
  using Node_t =
#if DUNE_VERSION_LT(DUNE_FUNCTIONS,2,7)
    typename PB::template Node<TP>;
#else
    typename PB::Node;
#endif

  // dune version independent extraction of node indexset type from preBasis
  template <class PB, class TP>
  using NodeIndexSet_t =
#if DUNE_VERSION_LT(DUNE_FUNCTIONS,2,7)
    typename PB::template IndexSet<TP>;
#else
    typename PB::IndexSet;
#endif

  // dune version independent creation of node from preBasis
  template <class PB, class TP>
  auto makeNode(PB const& preBasis, TP const& treePath)
  {
#if DUNE_VERSION_LT(DUNE_FUNCTIONS,2,7)
    return preBasis.node(treePath);
#else
    DUNE_UNUSED_PARAMETER(treePath);
    return preBasis.makeNode();
#endif
  }

  // dune version independent creation of node from preBasis
  template <class PB, class TP>
  auto makeNodeIndexSet(PB const& preBasis, TP const& treePath)
  {
#if DUNE_VERSION_LT(DUNE_FUNCTIONS,2,7)
    return preBasis.indexSet(treePath);
#else
    DUNE_UNUSED_PARAMETER(treePath);
    return preBasis.makeIndexSet();
#endif
  }

} // end namespace AMDiS
