#pragma once

#include <type_traits>

#include <amdis/localoperators/FirstOrderTestPartialTrial.hpp>

namespace AMDiS
{
  /**
   * \addtogroup operators
   * @{
   **/

  namespace tag
  {
    struct partialtest
    {
      std::size_t comp;
    };
  }


  /// first-order operator \f$ \langle\partial_i\psi, c\rangle \f$

  template <class LC, class GridFct>
  class GridFunctionOperator<tag::partialtest, LC, GridFct>
      : public GridFunctionOperatorBase<GridFunctionOperator<tag::partialtest, LC, GridFct>, LC, GridFct>
  {
    using Self = GridFunctionOperator;
    using Super = GridFunctionOperatorBase<Self, LC, GridFct>;

    static_assert( static_size_v<typename GridFct::Range> == 1, "Expression must be of scalar type." );

  public:
    GridFunctionOperator(tag::partialtest tag, GridFct const& expr)
      : Super(expr, 1)
      , comp_(tag.comp)
    {}

    template <class CG, class Node, class Vec>
    void getElementVector(CG const& contextGeo, Node const& node, Vec& elementVector)
    {
      static_assert(Node::isLeaf, "Operator can be applied to Leaf-Nodes only.");

      auto const& quad = this->getQuadratureRule(contextGeo.type(), node);
      auto localBasis = makeLocalToGlobalBasisAdapter(node, contextGeo.geometry());

      for (std::size_t iq = 0; iq < quad.size(); ++iq) {
        // Position of the current quadrature point in the reference element
        decltype(auto) local = contextGeo.local(quad[iq].position());

        // The multiplicative factor in the integral transformation formula
        auto dx = contextGeo.integrationElement(quad[iq].position()) * quad[iq].weight();
        dx *= Super::coefficient(local);

        // Compute the shape function gradients on the real element
        auto const& partial = localBasis.partialsAt(local, comp_);

        for (std::size_t j = 0; j < localBasis.size(); ++j) {
          const auto local_j = node.localIndex(j);
          elementVector[local_j] += dx * partial[j];
        }
      }
    }

  private:
    std::size_t comp_;
  };

  /** @} **/

} // end namespace AMDiS
